@@ -54,6 +54,7 @@ struct camera_info {
 
 struct device_info {
         const char *media_dev_name; // owned by camera config
+        const char *dev_name; // owned by camera config
 
         MPDevice *device;
 
@@ -132,8 +133,10 @@ setup_camera(MPDeviceList **device_list, const struct mp_camera_config *config)
         // Find device info
         size_t device_index = 0;
         for (; device_index < num_devices; ++device_index) {
-                if (strcmp(config->media_dev_name,
-                           devices[device_index].media_dev_name) == 0) {
+                if ((strcmp(config->media_dev_name,
+                            devices[device_index].media_dev_name) == 0) &&
+                    (strcmp(config->dev_name, devices[device_index].dev_name) ==
+                     0)) {
                         break;
                 }
         }
@@ -144,8 +147,9 @@ setup_camera(MPDeviceList **device_list, const struct mp_camera_config *config)
                 // Initialize new device
                 struct device_info *info = &devices[device_index];
                 info->media_dev_name = config->media_dev_name;
-                info->device = mp_device_list_find_remove(device_list,
-                                                          info->media_dev_name);
+                info->dev_name = config->dev_name;
+                info->device = mp_device_list_find_remove(
+                        device_list, info->media_dev_name, info->dev_name);
                 if (!info->device) {
                         g_printerr("Could not find /dev/media* node matching '%s'\n",
                                    info->media_dev_name);
@@ -273,18 +277,16 @@ setup_camera(MPDeviceList **device_list, const struct mp_camera_config *config)
 static void
 setup(MPPipeline *pipeline, const void *data)
 {
-        MPDeviceList *device_list = mp_device_list_new();
-
         for (size_t i = 0; i < MP_MAX_CAMERAS; ++i) {
                 const struct mp_camera_config *config = mp_get_camera_config(i);
                 if (!config) {
                         break;
                 }
 
+                MPDeviceList *device_list = mp_device_list_new();
                 setup_camera(&device_list, config);
+                mp_device_list_free(device_list);
         }
-
-        mp_device_list_free(device_list);
 }
 
 static void
