@@ -22,7 +22,6 @@ static const float colormatrix_srgb[] = { 3.2409, -1.5373, -0.4986, -0.9692, 1.8
 static MPPipeline *pipeline;
 
 static char burst_dir[23];
-static char processing_script[512];
 
 static volatile bool is_capturing = false;
 static volatile int frames_processed = 0;
@@ -31,7 +30,7 @@ static volatile int frames_received = 0;
 static const struct mp_camera_config *camera;
 static int camera_rotation;
 
-static MPCameraMode mode;
+static MPMode mode;
 
 static int burst_length;
 static int captures_remaining = 0;
@@ -51,10 +50,12 @@ static int gain_max;
 static bool exposure_is_manual;
 static int exposure;
 
-static bool save_dng;
+static bool flash_enabled;
 
 static char capture_fname[255];
 
+static GSettings *settings;
+
 static void
 register_custom_tiff_tags(TIFF *tif)
 {
@@ -75,8 +76,75 @@ register_custom_tiff_tags(TIFF *tif)
                            sizeof(custom_fields) / sizeof(custom_fields[0]));
 }
 
-static bool
-find_processor(char *script)
+void
+mp_process_find_all_processors(GtkListStore *store)
+{
+        GtkTreeIter iter;
+        char buffer[512];
+        // Find all the original postprocess.sh locations
+
+        // Check postprocess.sh in the current working directory
+        if (access("./data/postprocess.sh", F_OK) != -1) {
+                gtk_list_store_insert(store, &iter, -1);
+                gtk_list_store_set(store,
+                                   &iter,
+                                   0,
+                                   "./data/postprocess.sh",
+                                   1,
+                                   "(cwd) postprocess.sh",
+                                   -1);
+        }
+
+        // Check for a script in XDG_CONFIG_HOME
+        sprintf(buffer, "%s/megapixels/postprocess.sh", g_get_user_config_dir());
+        if (access(buffer, F_OK) != -1) {
+                gtk_list_store_insert(store, &iter, -1);
+                gtk_list_store_set(
+                        store, &iter, 0, buffer, 1, "(user) postprocess.sh", -1);
+        }
+
+        // Check user overridden /etc/megapixels/postprocessor.sh
+        sprintf(buffer, "%s/megapixels/postprocess.sh", SYSCONFDIR);
+        if (access(buffer, F_OK) != -1) {
+                gtk_list_store_insert(store, &iter, -1);
+                gtk_list_store_set(
+                        store, &iter, 0, buffer, 1, "(system) postprocess.sh", -1);
+        }
+
+        // Check user overridden /usr/share/megapixels/postprocessor.sh
+        sprintf(buffer, "%s/megapixels/postprocess.sh", DATADIR);
+        if (access(buffer, F_OK) != -1) {
+                gtk_list_store_insert(store, &iter, -1);
+                gtk_list_store_set(
+                        store, &iter, 0, buffer, 1, "(built-in) postprocess.sh", -1);
+        }
+
+        // Find extra packaged postprocessor scripts
+        // These should be packaged in
+        // /usr/share/megapixels/postprocessor.d/executable
+        sprintf(buffer, "%s/megapixels/postprocessor.d", DATADIR);
+        DIR *d;
+        struct dirent *dir;
+        d = opendir(buffer);
+        if (d) {
+                while ((dir = readdir(d)) != NULL) {
+                        if (dir->d_name[0] == '.') {
+                                continue;
+                        }
+                        sprintf(buffer,
+                                "%s/megapixels/postprocessor.d/%s",
+                                DATADIR,
+                                dir->d_name);
+                        gtk_list_store_insert(store, &iter, -1);
+                        gtk_list_store_set(
+                                store, &iter, 0, buffer, 1, dir->d_name, -1);
+                }
+                closedir(d);
+        }
+}
+
+bool
+mp_process_find_processor(char *script)
 {
         char filename[] = "postprocess.sh";
 
@@ -116,11 +184,7 @@ static void
 setup(MPPipeline *pipeline, const void *data)
 {
         TIFFSetTagExtender(register_custom_tiff_tags);
-
-        if (!find_processor(processing_script)) {
-                g_printerr("Could not find any post-process script\n");
-                exit(1);
-        }
+        settings = g_settings_new("org.postmarketos.Megapixels");
 }
 
 void
@@ -174,6 +238,47 @@ mp_process_pipeline_buffer_get_texture_id(MPProcessPipelineBuffer *buf)
         return buf->texture_id;
 }
 
+static void
+repack_image_sequencial(const uint8_t *src_buf, uint8_t *dst_buf, MPMode *mode)
+{
+        uint16_t pixels[4];
+        uint32_t row_length =
+                mp_pixel_format_width_to_bytes(mode->pixel_format, mode->width);
+        uint32_t padding_bytes =
+                mp_pixel_format_width_to_padding(mode->pixel_format, mode->width);
+        size_t si = 0;
+
+        // Image data must be 10-bit packed
+        assert(mp_pixel_format_bits_per_pixel(mode->pixel_format) == 10);
+
+        /*
+         * Repack 40 bits stored in sensor format into sequencial format
+         *
+         * src_buf: 11111111 22222222 33333333 44444444 11223344 ...
+         * dst_buf: 11111111 11222222 22223333 33333344 44444444 ...
+         */
+        for (size_t i = 0; i < row_length * mode->height; i += 5) {
+                // Skip padding bytes in source buffer
+                if (i && i % row_length == 0)
+                        si += padding_bytes;
+
+                /* Extract pixels from packed sensor format */
+                pixels[0] = (src_buf[si] << 2) | (src_buf[si + 4] >> 6);
+                pixels[1] = (src_buf[si + 1] << 2) | (src_buf[si + 4] >> 4 & 0x03);
+                pixels[2] = (src_buf[si + 2] << 2) | (src_buf[si + 4] >> 2 & 0x03);
+                pixels[3] = (src_buf[si + 3] << 2) | (src_buf[si + 4] & 0x03);
+
+                /* Pack pixels into sequencial format */
+                dst_buf[i] = (pixels[0] >> 2 & 0xff);
+                dst_buf[i + 1] = (pixels[0] << 6 & 0xff) | (pixels[1] >> 4 & 0x3f);
+                dst_buf[i + 2] = (pixels[1] << 4 & 0xff) | (pixels[2] >> 6 & 0x0f);
+                dst_buf[i + 3] = (pixels[2] << 2 & 0xff) | (pixels[3] >> 8 & 0x03);
+                dst_buf[i + 4] = (pixels[3] & 0xff);
+
+                si += 5;
+        }
+}
+
 static GLES2Debayer *gles2_debayer = NULL;
 
 static GdkGLContext *context;
@@ -224,14 +329,9 @@ init_gl(MPPipeline *pipeline, GdkSurface **surface)
                 check_gl();
         }
 
-        gles2_debayer = gles2_debayer_new(MP_PIXEL_FMT_BGGR8);
-        check_gl();
-
         glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
         check_gl();
 
-        gles2_debayer_use(gles2_debayer);
-
         for (size_t i = 0; i < NUM_BUFFERS; ++i) {
                 glGenTextures(1, &output_buffers[i].texture_id);
                 glBindTexture(GL_TEXTURE_2D, output_buffers[i].texture_id);
@@ -297,7 +397,9 @@ process_image_for_preview(const uint8_t *image)
         glTexImage2D(GL_TEXTURE_2D,
                      0,
                      GL_LUMINANCE,
-                     mode.width,
+                     mp_pixel_format_width_to_bytes(mode.pixel_format, mode.width) +
+                             mp_pixel_format_width_to_padding(mode.pixel_format,
+                                                              mode.width),
                      mode.height,
                      0,
                      GL_LUMINANCE,
@@ -462,9 +564,14 @@ process_image_for_capture(const uint8_t *image, int count)
         static const short cfapatterndim[] = { 2, 2 };
         TIFFSetField(tif, TIFFTAG_CFAREPEATPATTERNDIM, cfapatterndim);
 #if (TIFFLIB_VERSION < 20201219) && !LIBTIFF_CFA_PATTERN
-        TIFFSetField(tif, TIFFTAG_CFAPATTERN, "\002\001\001\000"); // BGGR
+        TIFFSetField(tif,
+                     TIFFTAG_CFAPATTERN,
+                     mp_pixel_format_cfa_pattern(mode.pixel_format));
 #else
-        TIFFSetField(tif, TIFFTAG_CFAPATTERN, 4, "\002\001\001\000"); // BGGR
+        TIFFSetField(tif,
+                     TIFFTAG_CFAPATTERN,
+                     4,
+                     mp_pixel_format_cfa_pattern(mode.pixel_format));
 #endif
         printf("TIFF version %d\n", TIFFLIB_VERSION);
         int whitelevel = camera->whitelevel;
@@ -480,10 +587,21 @@ process_image_for_capture(const uint8_t *image, int count)
         TIFFCheckpointDirectory(tif);
         printf("Writing frame to %s\n", fname);
 
+        uint8_t *output_image = (uint8_t *)image;
+
+        // Repack 10-bit image from sensor format into a sequencial format
+        if (mp_pixel_format_bits_per_pixel(mode.pixel_format) == 10) {
+                output_image = malloc(mp_pixel_format_width_to_bytes(
+                                              mode.pixel_format, mode.width) *
+                                      mode.height);
+
+                repack_image_sequencial(image, output_image, &mode);
+        }
+
         for (int row = 0; row < mode.height; row++) {
                 TIFFWriteScanline(
                         tif,
-                        (void *)image +
+                        (void *)output_image +
                                 (row * mp_pixel_format_width_to_bytes(
                                                mode.pixel_format, mode.width)),
                         row,
@@ -491,6 +609,9 @@ process_image_for_capture(const uint8_t *image, int count)
         }
         TIFFWriteDirectory(tif);
 
+        if (output_image != image)
+                free(output_image);
+
         // Add an EXIF block to the tiff
         TIFFCreateEXIFDirectory(tif);
         // 1 = manual, 2 = full auto, 3 = aperture priority, 4 = shutter priority
@@ -510,7 +631,16 @@ process_image_for_capture(const uint8_t *image, int count)
                         gain - 1, 0, gain_max, camera->iso_min, camera->iso_max);
                 TIFFSetField(tif, EXIFTAG_ISOSPEEDRATINGS, 1, &isospeed);
         }
-        TIFFSetField(tif, EXIFTAG_FLASH, 0);
+        if (!camera->has_flash) {
+                // No flash function
+                TIFFSetField(tif, EXIFTAG_FLASH, 0x20);
+        } else if (flash_enabled) {
+                // Flash present and fired
+                TIFFSetField(tif, EXIFTAG_FLASH, 0x1);
+        } else {
+                // Flash present but not fired
+                TIFFSetField(tif, EXIFTAG_FLASH, 0x0);
+        }
 
         TIFFSetField(tif, EXIFTAG_DATETIMEORIGINAL, datetime);
         TIFFSetField(tif, EXIFTAG_DATETIMEDIGITIZED, datetime);
@@ -587,6 +717,9 @@ process_capture_burst(GdkTexture *thumb)
                         timestamp);
         }
 
+        bool save_dng = g_settings_get_boolean(settings, "save-raw");
+        char *postprocessor = g_settings_get_string(settings, "postprocessor");
+
         char save_dng_s[2] = "0";
         if (save_dng) {
                 save_dng_s[0] = '1';
@@ -600,7 +733,7 @@ process_capture_burst(GdkTexture *thumb)
         g_autoptr(GError) error = NULL;
         GSubprocess *proc = g_subprocess_new(G_SUBPROCESS_FLAGS_STDOUT_PIPE,
                                              &error,
-                                             processing_script,
+                                             postprocessor,
                                              burst_dir,
                                              capture_fname,
                                              save_dng_s,
@@ -623,8 +756,10 @@ process_image(MPPipeline *pipeline, const MPBuffer *buffer)
         clock_t t1 = clock();
 #endif
 
-        size_t size = mp_pixel_format_width_to_bytes(mode.pixel_format, mode.width) *
-                      mode.height;
+        size_t size =
+                (mp_pixel_format_width_to_bytes(mode.pixel_format, mode.width) +
+                 mp_pixel_format_width_to_padding(mode.pixel_format, mode.width)) *
+                mode.height;
         uint8_t *image = malloc(size);
         memcpy(image, buffer->data, size);
         mp_io_pipeline_release_buffer(buffer->index);
@@ -718,7 +853,7 @@ mp_process_pipeline_capture()
 }
 
 static void
-on_output_changed()
+on_output_changed(bool format_changed)
 {
         output_buffer_width = mode.width / 2;
         output_buffer_height = mode.height / 2;
@@ -744,6 +879,17 @@ on_output_changed()
 
         glBindTexture(GL_TEXTURE_2D, 0);
 
+        // Create new gles2_debayer on format change
+        if (format_changed) {
+                if (gles2_debayer)
+                        gles2_debayer_free(gles2_debayer);
+
+                gles2_debayer = gles2_debayer_new(mode.pixel_format);
+                check_gl();
+
+                gles2_debayer_use(gles2_debayer);
+        }
+
         gles2_debayer_configure(
                 gles2_debayer,
                 output_buffer_width,
@@ -766,11 +912,12 @@ mod(int a, int b)
 static void
 update_state(MPPipeline *pipeline, const struct mp_process_pipeline_state *state)
 {
-        const bool output_changed =
-                !mp_camera_mode_is_equivalent(&mode, &state->mode) ||
-                preview_width != state->preview_width ||
-                preview_height != state->preview_height ||
-                device_rotation != state->device_rotation;
+        const bool output_changed = !mp_mode_is_equivalent(&mode, &state->mode) ||
+                                    preview_width != state->preview_width ||
+                                    preview_height != state->preview_height ||
+                                    device_rotation != state->device_rotation;
+
+        const bool format_changed = mode.pixel_format != state->mode.pixel_format;
 
         camera = state->camera;
         mode = state->mode;
@@ -781,7 +928,6 @@ update_state(MPPipeline *pipeline, const struct mp_process_pipeline_state *state
         device_rotation = state->device_rotation;
 
         burst_length = state->burst_length;
-        save_dng = state->save_dng;
 
         // gain_is_manual = state->gain_is_manual;
         gain = state->gain;
@@ -793,7 +939,7 @@ update_state(MPPipeline *pipeline, const struct mp_process_pipeline_state *state
         if (output_changed) {
                 camera_rotation = mod(camera->rotate - device_rotation, 360);
 
-                on_output_changed();
+                on_output_changed(format_changed);
         }
 
         struct mp_main_state main_state = {
