@@ -1,12 +1,14 @@
 #pragma once
 
+#include "camera.h"
 #include "camera_config.h"
+#include <gtk/gtk.h>
 
 typedef struct _GdkSurface GdkSurface;
 
 struct mp_process_pipeline_state {
         const struct mp_camera_config *camera;
-        MPCameraMode mode;
+        MPMode mode;
 
         int burst_length;
 
@@ -25,9 +27,12 @@ struct mp_process_pipeline_state {
         bool has_auto_focus_continuous;
         bool has_auto_focus_start;
 
-        bool save_dng;
+        bool flash_enabled;
 };
 
+bool mp_process_find_processor(char *script);
+void mp_process_find_all_processors(GtkListStore *store);
+
 void mp_process_pipeline_start();
 void mp_process_pipeline_stop();
 void mp_process_pipeline_sync();
